/*

Testprogram fuer den Analog Devices TMP03/04 Temperatursensor
im TO-92 Gehaeuse

URL: http://www.analog.com/UploadedFiles/Data_Sheets/14820945TMP03_4_a.pdf

Date:           2008-02-15, Version 1.00

Compiler:       CCS V3.236 or later
Target:         PIC 18F8722
Testschaltung:  EPM8722 R3 Experimentierplatine
Ausgabe:        RS-232 mit 9600, N,8,1

Wirkungsweise des TMP04:
========================
(Frei uebersetzt vom Datenblatt)

Der Ausgang des TMP04 ist eine Rechteckspannung mit einer nominellen
Taktfrequenz von 35Hz (+/-20%) bei 25 C. Das Ausgangsformat laesst sich
mit dem Capture Timer von einem Mikro leicht dekodieren wie hier beschrieben:

        |<-- T1 -->|<------- T2 -------->|
         __________                       _________
________|          |_____________________|

Temperature in (C) = 235 - ( 400 x T1 / T2 )

Da beide Perioden nacheinander mit demselben Takt gemessen werden,
ist das Resultat ratiometrisch, unabhaengig vom internen Takt des
TMP04 oder von dem Takt des Zaehlers im Mikro.

Anschluss des TMP04 am Mikro:
=============================
(TO-92 Gehaeuse mit Beschriftung nach oben gezeigt)

 _________
|         |\---- Masse (GND)
| TMP03/4 |-)--- +5V (V+)
|_________|/---- Rechtecksignal (Dout) zum RC1 Anschluss des PIC Mikro


Der TMP03 kann auch verwendet werden, braucht aber einen Pull-up Widerstand
wegen des Open Drain Ausgangs.

*/

#include <18F8722.h>
#include "types.h"

#use fast_io(c)
#use delay(clock=16000000)
#use rs232 (baud=9600, xmit=PIN_C6, rcv=PIN_C7)

#fuses HS, NOWDT, NOPROTECT, NOLVP

#define PORT_C_MASK 0b10000010

uint8_t _temperature_update_clock=0;

uint32_t _ccp2_val[2];
uint8_t _timer1_rollover=0;

uint16_t _tmp04_t1=0,
         _tmp04_t2=0;


// Fuer Ueberlaeufe des Timer1
#int_timer1
void timer1_isr (void)
{
    ++_timer1_rollover;
}



// Misst die TMP04 Rechteckwerte T1 und T2 Werte mit 1us Aufloesung
#int_ccp2
void ccp1_isr (void)
{
    static bit_t TMP04_EDGE_FLAG = FALSE;
    static uint32_t temp;

    temp = (uint32_t) CCP_2 + ((uint32_t) _timer1_rollover * 65536);

    // Capture Interrupt durch Vorderflanke verursacht
    if(TMP04_EDGE_FLAG == TRUE)
    {
        setup_CCP2(CCP_CAPTURE_FE); // Schalte den CCP wieder auf die andere Flanke um
        TMP04_EDGE_FLAG = FALSE;
        _ccp2_val[0] = temp;
        _tmp04_t2 = (uint16_t) ((_ccp2_val[0] - _ccp2_val[1]));
    }
    else
    // Capture Interrupt durch Hinterflanke verursacht
    {
        setup_CCP2(CCP_CAPTURE_RE); // Schalte den CCP wieder auf die andere Flanke um
        TMP04_EDGE_FLAG = TRUE;
        _ccp2_val[1] = temp;
        _tmp04_t1 = (uint16_t) ((_ccp2_val[1] - _ccp2_val[0]));
    }
}



void main()
{
    // PortC Konfigurierung (Pin C1 = Eingang)
    set_tris_c(PORT_C_MASK);

    // Einrichtung des Capture Eingang (RC1 Pin) fuer Erkennung der Vorderflanke der
    // TMP04 Rechteckspannung beim ersten Betrieb
    setup_CCP2(CCP_CAPTURE_RE );
    // Konfigurierung des TIMER 1 Zaehlers mit einer Aufloesung von 1us
    setup_timer_1(T1_INTERNAL | T1_DIV_BY_4);

    set_timer1(0);

    enable_interrupts(INT_TIMER1);
    enable_interrupts(INT_CCP2);
    enable_interrupts(GLOBAL);

    printf("\r\nAnalog Devices TMP03/04 Temperatursensor Test Program");
    printf("\r\nTemperaturwert wird jede Sekunden ausgegeben\r\n");

    for( ; ; )
    {
        delay_ms(1000);
        printf( "\nT1 = %lu us, T2 = %lu us, ", _tmp04_t1, _tmp04_t2);
        printf ("%4.1f DEG C ", 235.0 - ( 400.0 * (float) _tmp04_t1 / (float) _tmp04_t2));
    }
}















